﻿using System.Collections.ObjectModel;
using GalaSoft.MvvmLight;
using GalaSoft.MvvmLight.Command;
using MvvmLight5.Model;

namespace MvvmLight5.ViewModel
{
   /// <summary>
   /// This class contains properties that the main View can data bind to.
   /// <para>
   /// Use the <strong>mvvminpc</strong> snippet to add bindable properties to this ViewModel.
   /// </para>
   /// <para>
   /// You can also use Blend to data bind with the tool's support.
   /// </para>
   /// <para>
   /// See http://www.galasoft.ch/mvvm/getstarted
   /// </para>
   /// </summary>
   public class MainViewModel : ViewModelBase
   {
      /// <summary>
      /// The <see cref="Customers" /> property's name.
      /// </summary>
      public const string CustomersPropertyName = "Customers";

      /// <summary>
      /// Gets the Customers property.
      /// TODO Update documentation:
      /// Changes to that property's value raise the PropertyChanged event.
      /// This property's value is broadcasted by the Messenger's default instance when it changes.
      /// </summary>
      public ObservableCollection<Customer> Customers
      {
         get
         {
            var customerCollection = new CustomerCollection();
            return customerCollection.Customers;
         }
      }

      public RelayCommand<Customer> DetailsPageCommand
      {
         get;
         private set;
      }

      public string ApplicationTitle
      {
         get
         {
            return "MVVM LIGHT";
         }
      }

      public string PageName
      {
         get
         {
            return "My page:";
         }
      }

      public string Welcome
      {
         get
         {
            return "Welcome to MVVM Light";
         }
      }

      /// <summary>
      /// Initializes a new instance of the MainViewModel class.
      /// </summary>
      public MainViewModel()
      {
         if ( IsInDesignMode )
         {
            // Code runs in Blend --> create design time data.
         }
         else
         {
            // Code runs "for real"
         }
         DetailsPageCommand = new RelayCommand<Customer>( ( msg ) => GoToDetailsPage( msg ) );
      }

      private object GoToDetailsPage( Customer msg )
      {
         System.Windows.MessageBox.Show( "Go to details page with: " + msg.First + " " + msg.Last );
         return null;
      }

      ////public override void Cleanup()
      ////{
      ////    // Clean up if needed

      ////    base.Cleanup();
      ////}
   }
}